# Generated by Django 4.2.5 on 2023-09-25 08:24

from django.db import migrations, models

import quantityfield.fields


class Migration(migrations.Migration):
    dependencies = [
        ("dummyapp", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="OffsetUnitFloatFieldSaveModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=20)),
                (
                    "weight",
                    quantityfield.fields.QuantityField(
                        base_units="degC", unit_choices=["degC"]
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]
